@@ -19,6 +19,7 @@ from typing import List, Tuple, Optional, Union, Type
 from collections import OrderedDict
 import sys
 import zeep
+from itertools import zip_longest
 sys.path.append("..")
 try:
     from custom_exceptions.salesforce_exceptions import *
@@ -567,7 +568,7 @@ class Sf:
         return await async_query(query, excludedFields)
     
     async def async_queries(self, queries: List[str]):
-        async_queries = async_wrap(self.sf.query)
+        async_queries = async_wrap(self.sf.query_all)
         result_list = await asyncio.gather(*[async_queries(query) for query in queries])
         result_pd_list = [transform_sf_result_set_rec(result['records']) for result in result_list]
         return result_pd_list
@@ -682,6 +683,54 @@ class Sf:
         result_list = [item for sublist in result_list for item in sublist]
         return result_list
 
+
+    async def retrieve_metadata(self, metadata_type: list[str]):
+        
+        """Providing a list of metadata type in accordance with the Salesforce Metadata API
+        https://developer.salesforce.com/docs/atlas.en-us.api_meta.meta/api_meta/meta_types_list.htm
+
+        Raises:
+            Exception: Metadata Name Mismatch, this may be caused by async operation returning metadata in unexpected order, or one of the metadata types requestd cannot be retreived properly
+
+        Returns:
+            pd.DataFrame: Returns the full detailed metadata dataframe of all the metadata types requested
+        """
+        metadata_list_async = async_wrap(self.metadata_list)
+        task_list = []
+        for group in self.grouper(metadata_type, 3):
+            meta_list = [value for value in group if value is not None]
+            task_list.append(meta_list)
+        result_list = await asyncio.gather(*[metadata_list_async(metadata_type_list=task ) for task in task_list])
+        result_list = [item for sublist in result_list for item in sublist]
+        metadata_info = pd.DataFrame(zeep.helpers.serialize_object(result_list))
+        
+        
+        metadata_read_async = async_wrap(self.metadata_read)
+        
+        detailed_metadata = await asyncio.gather(*[metadata_read_async(metadata_type=result['type'], fullName=result['fullName']) for result in result_list])
+        
+        
+        combined_metadata = []
+        for item in zip(detailed_metadata, metadata_info.iterrows()):
+            
+            x_data = item[0].loc[0]
+            x_info = item[1][1]
+
+
+            if x_info['fullName'] == x_data['fullName']:
+                x_data.drop('fullName', inplace=True)
+            else:
+                raise Exception('Metadata Name Mismatch, this may be caused by async operation returning metadata in unexpected order, or one of the metadata types requestd cannot be retreived properly')
+            x_info.name= 0
+            
+            combined_row = pd.DataFrame(pd.concat([x_info, x_data], axis=0)).T
+            combined_metadata.append(combined_row)
+
+        combined_result_df = pd.concat(combined_metadata, axis=0).reset_index(drop=True)
+        
+        return combined_result_df
+
+        
     async def field_permission_bulk_check(self, objects: list, ) -> dict[str, pd.DataFrame]:
         async_field_permission_check = async_wrap(self.field_permission_check)
         result_list = await asyncio.gather(*[async_field_permission_check( object ) for object in objects])
@@ -1427,7 +1476,22 @@ class Sf:
                 return r.json()
         else:
             raise Exception('API error when calling %s : %s' % (r.url, r.content))
-
+        
+    def grouper(iterable, n, *, incomplete='fill', fillvalue=None):
+        "Collect data into non-overlapping fixed-length chunks or blocks"
+        # grouper('ABCDEFG', 3, fillvalue='x') --> ABC DEF Gxx
+        # grouper('ABCDEFG', 3, incomplete='strict') --> ABC DEF ValueError
+        # grouper('ABCDEFG', 3, incomplete='ignore') --> ABC DEF
+        args = [iter(iterable)] * n
+        if incomplete == 'fill':
+            return zip_longest(*args, fillvalue=fillvalue)
+        if incomplete == 'strict':
+            return zip(*args, strict=True)
+        if incomplete == 'ignore':
+            return zip(*args)
+        else:
+            raise ValueError('Expected fill, strict, or ignore')
+        
 
     def join(self, 
                 dataframe, 
